macroscript AttachToSurface
category:"BodyulCG"
tooltip:"AttachToSurface"
(
	local screenPos
	local isOpen = false
	
	rollout AttachToSurface "AttachToSurface" width:200
	(
		local nodeCb
		local attachedObjects = #()
		
		fn isGeometry obj =
			return isKindOf obj geometryclass
		
		fn attachFilter obj =
			return finditem attachedObjects obj == 0 and AttachToSurface.pbtnTarget.object != obj;
		
		group "Attach To:"
		(
			pickbutton pbtnTarget "Pick Object" width:180 height:28 autodisplay:true filter:isGeometry align:#center
		)
		
		group "Attached Objects"
		(
			checkbox chkAlignToSurface "Align To Surface" align:#center
			multilistbox mlbxAttachedObjects offset:[0, 2]
			pickbutton pbtnAttach "Attach" width:56 height:24 align:#left filter:attachFilter across:3 enabled:false
			button btnAttachList "..." width:24 height:24 iconName:"MainUI/SelectByName" iconsize:[18, 18] enabled:false tooltip:"Attach By List"
			button btnDetach "Detach" width:56 height:24 enabled:false
		)
		
		group "About"
		(
			label lblAbout "AttachToSurface v1.1"
			hyperlink hpUrl "bodyulcg.com" address:"https://bodyulcg.com" color:(color 0 250 210) hovercolor:(color 0 250 210) visitedcolor:(color 0 250 210) align:#center
		)
		
		fn closestPoint obj pos = 
		(
			mpi = MeshProjIntersect()
			mpi.setNode obj
			mpi.build()
		 
			mpi.closestFace pos doubleSided:true
		 
			HitPos = mpi.GetHitPos()
			HitFace = mpi.GetHitFace()
			HitBary = mpi.GetHitBary()
			mpi.Free()
			
			return #(HitPos, HitFace, HitBary)
		)
		
		fn attach obj target =
		(
			local cpt = closestPoint target obj.pos
			local ctrl = Attachment node:target align:chkAlignToSurface.checked
			local akey = AttachCtrl.addNewKey ctrl 0
			akey.face = cpt[2]
			akey.coord = cpt[3]
			obj.position.controller = ctrl
		)
		
		fn detach obj =
		(
			local ctrl = Position_XYZ()
			ctrl.value = obj.position
			obj.position.controller = ctrl
		)
		
		fn refreshList =
		(
			local items = #()
			local target = pbtnTarget.object

			free attachedObjects
			
			if isValidNode target then
				for i in refs.dependentNodes target do
					if isProperty i #position and classof i.position.controller == Attachment and i.position.controller.node == target do
					(
						append attachedObjects i
						append items i.name
					)

			mlbxAttachedObjects.items = items
		)
		
		fn updateUI =
		(
			local isValidTarget = isValidNode pbtnTarget.object
			pbtnAttach.enabled = isValidTarget
			btnAttachList.enabled = isValidTarget
			btnDetach.enabled = mlbxAttachedObjects.selection.numberset > 0
		)
		
		fn nodeCallback e n =
			pbtnTarget.picked (pbtnTarget.object = pbtnTarget.object);

		on AttachToSurface open do
		(
			if screenPos != undefined then
				setDialogPos AttachToSurface screenPos;

			nodeCb = nodeeventcallback added:nodeCallback deleted:nodeCallback namechanged:nodeCallback
			btnAttachList.pos.x = pbtnAttach.pos.x + 58
		)
		
		on pbtnTarget picked obj do
		(
			refreshList()
			
			if attachedObjects.count > 0 then
				chkAlignToSurface.checked = attachedObjects[1].position.controller.align;
				
			local isValidTarget = isValidNode obj
			pbtnAttach.enabled = isValidTarget
			btnAttachList.enabled = isValidTarget
		)
		
		on chkAlignToSurface changed state do
			for i in attachedObjects where isValidNode i do
				i.position.controller.align = state;
		
		on mlbxAttachedObjects selectionEnd do
			updateUI();
		
		on mlbxAttachedObjects doubleclicked sel do
			if isValidNode attachedObjects[sel] then
				(if keyboard.controlPressed then selectmore else select) attachedObjects[sel]
		
		on pbtnAttach picked obj do
		(
			if not isValidNode pbtnTarget.object then
				return ok;
				
			attach obj pbtnTarget.object
			refreshList()
		)
		
		on btnAttachList pressed do
		(
			if not isValidNode pbtnTarget.object then
				return ok;
			
			local objs = selectbyname filter:attachFilter
			
			if objs == undefined then 
				return ok;
			
			for i in objs do
				attach i pbtnTarget.object
			
			refreshList()
		)
		
		on btnDetach pressed do
		(
			local sel = mlbxAttachedObjects.selection as array
			
			for i = sel.count to 1 by -1 do
				if isValidNode attachedObjects[sel[i]] then
					detach attachedObjects[sel[i]];
			
			mlbxAttachedObjects.selection = #{}
			refreshList()
		)
		
		on AttachToSurface close do
		(
			screenPos = getDialogPos AttachToSurface
			nodeCb = undefined
			free attachedObjects
			gc light:true
			
			isOpen = false
			updateToolbarButtons()
		)
	)
	
	on execute do
	(
		if isOpen then
		(
			destroyDialog AttachToSurface
			isOpen = false
		)
		else
		(
			createDialog AttachToSurface
			isOpen = true
		)		
	)

	on isChecked do
		isOpen;

	on closeDialogs do
		destroyDialog AttachToSurface;
)